use cosmic_text::CacheKeyFlags;
#[cfg(feature = "databake")]
use databake::Bake;
use jotdown::{ListKind, OrderedListNumbering, OrderedListStyle};
#[cfg(feature = "serde")]
use serde::{Deserialize, Serialize};

#[derive(Clone, Debug)]
#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
#[cfg_attr(feature = "serde", serde(remote = "Option<ListKind>"))]
pub enum ListKindOption {
    Some(#[cfg_attr(feature = "serde", serde(with = "ListKindSerde"))] ListKind),
    None,
}

#[derive(Clone, Debug)]
#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
#[cfg_attr(feature = "serde", serde(remote = "jotdown::ListKind"))]
pub enum ListKindSerde {
    Unordered,
    Ordered {
        numbering: OrderedListNumbering,
        style: OrderedListStyle,
        start: u64,
    },
    Task,
}

#[derive(Clone, Debug)]
#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
#[cfg_attr(feature = "serde", serde(remote = "jotdown::OrderedListNumbering"))]
pub enum OrderedListNumberingSerde {
    Decimal,
    AlphaLower,
    AlphaUpper,
    RomanLower,
    RomanUpper,
}

#[derive(Clone, Debug)]
#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
#[cfg_attr(feature = "serde", serde(remote = "jotdown::OrderedListStyle"))]
pub enum OrderedListStyleSerde {
    Period,
    Paren,
    ParenParen,
}

#[derive(Clone, Debug)]
#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
#[cfg_attr(feature = "serde", serde(remote = "cosmic_text::Metrics"))]
#[cfg_attr(feature = "databake", derive(Bake))]
#[cfg_attr(feature = "databake", databake(path = cosmic_text))]
pub struct Metrics {
    pub font_size: f32,
    pub line_height: f32,
}

#[derive(Clone, Debug)]
#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
#[cfg_attr(feature = "serde", serde(remote = "cosmic_text::Family"))]
#[cfg_attr(feature = "databake", derive(Bake))]
#[cfg_attr(feature = "databake", databake(path = cosmic_text))]
pub enum Family<'a> {
    Name(&'a str),
    Serif,
    SansSerif,
    Cursive,
    Fantasy,
    Monospace,
}

impl<'a> From<cosmic_text::Family<'a>> for Family<'a> {
    fn from(value: cosmic_text::Family<'a>) -> Self {
        match value {
            cosmic_text::Family::Name(n) => Self::Name(n),
            cosmic_text::Family::Serif => Self::Serif,
            cosmic_text::Family::SansSerif => Self::SansSerif,
            cosmic_text::Family::Cursive => Self::Cursive,
            cosmic_text::Family::Fantasy => Self::Fantasy,
            cosmic_text::Family::Monospace => Self::Monospace,
        }
    }
}

#[derive(Clone, Debug)]
#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
#[cfg_attr(feature = "serde", serde(remote = "cosmic_text::Stretch"))]
#[cfg_attr(feature = "databake", derive(Bake))]
#[cfg_attr(feature = "databake", databake(path = cosmic_text))]
pub enum Stretch {
    UltraCondensed,
    ExtraCondensed,
    Condensed,
    SemiCondensed,
    Normal,
    SemiExpanded,
    Expanded,
    ExtraExpanded,
    UltraExpanded,
}

impl From<cosmic_text::Stretch> for Stretch {
    fn from(value: cosmic_text::Stretch) -> Self {
        match value {
            cosmic_text::Stretch::UltraCondensed => Self::UltraCondensed,
            cosmic_text::Stretch::ExtraCondensed => Self::ExtraCondensed,
            cosmic_text::Stretch::Condensed => Self::Condensed,
            cosmic_text::Stretch::SemiCondensed => Self::SemiCondensed,
            cosmic_text::Stretch::Normal => Self::Normal,
            cosmic_text::Stretch::SemiExpanded => Self::SemiExpanded,
            cosmic_text::Stretch::Expanded => Self::Expanded,
            cosmic_text::Stretch::ExtraExpanded => Self::ExtraExpanded,
            cosmic_text::Stretch::UltraExpanded => Self::UltraExpanded,
        }
    }
}

#[derive(Clone, Debug)]
#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
#[cfg_attr(feature = "serde", serde(remote = "cosmic_text::Style"))]
#[cfg_attr(feature = "databake", derive(Bake))]
#[cfg_attr(feature = "databake", databake(path = cosmic_text))]
pub enum Style {
    Normal,
    Italic,
    Oblique,
}

impl From<cosmic_text::Style> for Style {
    fn from(value: cosmic_text::Style) -> Self {
        match value {
            cosmic_text::Style::Normal => Style::Normal,
            cosmic_text::Style::Italic => Style::Italic,
            cosmic_text::Style::Oblique => Style::Oblique,
        }
    }
}

#[derive(Clone, Debug)]
#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
#[cfg_attr(feature = "serde", serde(remote = "cosmic_text::Weight"))]
#[cfg_attr(feature = "databake", derive(Bake))]
#[cfg_attr(feature = "databake", databake(path = cosmic_text))]
pub struct Weight(pub u16);

#[derive(Clone, Debug)]
#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
#[cfg_attr(feature = "serde", serde(remote = "cosmic_text::Color"))]
#[cfg_attr(feature = "databake", derive(Bake))]
#[cfg_attr(feature = "databake", databake(path = cosmic_text))]
pub struct Color(pub u32);

#[derive(Clone, Debug)]
#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
#[cfg_attr(feature = "serde", serde(remote = "Option<cosmic_text::Color>"))]
pub enum ColorOpt {
    Some(#[cfg_attr(feature = "serde", serde(with = "Color"))] cosmic_text::Color),
    None,
}

#[cfg(feature = "serde")]
mod cache_key_flags {
    use cosmic_text::CacheKeyFlags;
    use serde::{Deserialize, Deserializer, Serialize, Serializer};

    pub fn serialize<S>(_: &CacheKeyFlags, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer,
    {
        ().serialize(serializer)
    }

    pub fn deserialize<'de, D>(deserializer: D) -> Result<CacheKeyFlags, D::Error>
    where
        D: Deserializer<'de>,
    {
        let _: () = <()>::deserialize(deserializer)?;
        Ok(CacheKeyFlags::empty())
    }
}

#[derive(Clone, Debug)]
#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
#[cfg_attr(feature = "serde", serde(remote = "cosmic_text::Attrs"))]
pub struct AttrsSerde<'a> {
    #[cfg_attr(feature = "serde", serde(with = "ColorOpt"))]
    pub color_opt: Option<cosmic_text::Color>,
    #[cfg_attr(feature = "serde", serde(with = "Family"))]
    #[cfg_attr(feature = "serde", serde(borrow))]
    pub family: cosmic_text::Family<'a>,
    #[cfg_attr(feature = "serde", serde(with = "Stretch"))]
    pub stretch: cosmic_text::Stretch,
    #[cfg_attr(feature = "serde", serde(with = "Style"))]
    pub style: cosmic_text::Style,
    #[cfg_attr(feature = "serde", serde(with = "Weight"))]
    pub weight: cosmic_text::Weight,
    pub metadata: usize,
    #[cfg_attr(feature = "serde", serde(with = "cache_key_flags"))]
    pub cache_key_flags: CacheKeyFlags,
}

impl<'a> From<cosmic_text::Attrs<'a>> for AttrsSerde<'a> {
    fn from(value: cosmic_text::Attrs<'a>) -> Self {
        AttrsSerde {
            color_opt: value.color_opt,
            family: value.family,
            stretch: value.stretch,
            style: value.style,
            weight: value.weight,
            metadata: value.metadata,
            cache_key_flags: value.cache_key_flags,
        }
    }
}

#[cfg(feature = "databake")]
impl<'a> Bake for AttrsSerde<'a> {
    fn bake(&self, ctx: &databake::CrateEnv) -> databake::TokenStream {
        let color = self.color_opt.map(|c| Color(c.0)).bake(ctx);
        let family: Family = self.family.into();
        let family = family.bake(ctx);
        let stretch: Stretch = self.stretch.into();
        let stretch = stretch.bake(ctx);
        let style: Style = self.style.into();
        let style = style.bake(ctx);
        let weight: Weight = Weight(self.weight.0);
        let weight = weight.bake(ctx);
        let metadata = self.metadata.bake(ctx);
        databake::quote! {
            cosmic_text::Attrs {
                color_opt: #color,
                family: #family,
                stretch: #stretch,
                style: #style,
                weight: #weight,
                metadata: #metadata,
                cache_key_flags: cosmic_text::CacheKeyFlags::empty()
            }
        }
    }
}

#[derive(Clone, Debug)]
#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
#[cfg_attr(feature = "serde", serde(remote = "Option<cosmic_text::Align>"))]
pub enum AlignSerde {
    Some(#[cfg_attr(feature = "serde", serde(with = "Align"))] cosmic_text::Align),
    None,
}

#[derive(Clone, Debug)]
#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
#[cfg_attr(feature = "serde", serde(remote = "cosmic_text::Align"))]
#[cfg_attr(feature = "databake", derive(Bake))]
#[cfg_attr(feature = "databake", databake(path = cosmic_text))]
pub enum Align {
    Left,
    Right,
    Center,
    Justified,
    End,
}
